import unittest
import wgba

class TestAll(unittest.TestCase):
    def test_example_bw(self):
        assert 'hg19' in wgba.check_bigwig(path = "test/test.bw")

    def test_example_bed(self):
        assert 'grch38' in wgba.check_bed(path = "test/grch38.bed")



import pandas as pd
import pyBigWig

from .genomes import build_genomes


def check_bigwig(path: str, summary: bool = False, tol: int = 2, **other) -> list:
    bw = pyBigWig.open(path)

    consistent = {}
    for g in build_genomes():
        consistent[g.build] = {}
        for chrom, length in bw.chroms().items():
            if chrom not in g.chroms():
                consistent[g.build][chrom] = False
                continue

            if length > g[chrom]:
                consistent[g.build][chrom] = False
            else:
                consistent[g.build][chrom] = True

    genomes = report(path, consistent, summary=summary, tol=tol)
    return genomes


def check_bed(path: str, summary: bool = False, tol: int = 2, **other) -> list:
    consistent = {}
    genomes = build_genomes()
    for g in genomes:
        consistent[g.build] = {}
        for chrom in g.chroms():
            consistent[g.build][chrom] = True

    with open(path) as bed:
        for line in bed:
            chr, start, end = line.strip().split("\t")[0:3]
            for g in genomes:
                if chr in g.chroms():
                    if int(end) > g.size_of(chr):
                        consistent[g.build][chr] = False

    genomes = report(path, consistent, summary=summary, tol=tol)
    return genomes


def report(path: str, consistent: dict, summary: bool = False, tol: int = 2) -> list:
    genomes = []
    for g in consistent.keys():
        if summary:
            print(f"Not consistent with {g}:")
        n_not = 0
        for chrom in consistent[g].keys():
            if not consistent[g][chrom]:
                n_not += 1
                if summary:
                    print(f"\t{chrom}")

        if n_not < tol:
            genomes.append(g)

    if len(genomes) > 0:
        print(f"{path} is probably built on {', '.join(genomes)}.")
    else:
        print(f"{path} doesn't match any genomes that I know about.")

    return genomes



import pkg_resources
from shutil import copy


class Genome:
    def __init__(self, name):
        self.sizes = {}
        self.build = name
        path = self.genome_path(name)

        with open(path) as f:
            for line in f:
                chrom, length = line.strip().split()
                self.sizes[chrom] = int(length)

    def __getitem__(self, key):
        return self.sizes[key]

    def items(self):
        return self.sizes.items()

    def chroms(self):
        return [k for k in self.sizes.keys()]

    def size_of(self, chrom):
        return self.sizes[chrom]

    def genome_path(self, name):
        return pkg_resources.resource_filename("wgba.sizes", f"{name}.chrom.sizes")
        # Try except here for names not found


def build_genomes():
    genomes = []

    files = [
        i.replace(".chrom.sizes", "")
        for i in pkg_resources.resource_listdir("wgba.sizes", ".")
        if i not in ["__init__.py", "__pycache__"]
    ]
    for g in files:
        genomes.append(Genome(g))

    return genomes


def add_build(path):
    assert (
        ".chrom.sizes" in path
    ), "Your file name must begin with the build ID and end with .chrom.sizes"

    files = [
        i.replace(".chrom.sizes", "")
        for i in pkg_resources.resource_listdir("wgba.sizes", ".")
        if i not in ["__init__.py", "__pycache__"]
    ]
    folder = (
        "/".join(
            pkg_resources.resource_filename("wgba.sizes", files[0]).split("/")[:-1]
        )
        + "/"
    )

    copy(path, folder + path)

    print(
        f"WGBA: Successfully added '{path.split('/')[-1].replace('.chrom.sizes', '')}' to the list of known genomes."
    )



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
   name='wgba',
   version='1.0',
   python_requires='>3.6', 
   description='Which genome build again?',
   long_description=long_description,
   long_description_content_type="text/markdown",
   url="https://github.com/Chris1221/wgba",
   project_urls={
      "Bug Tracker": "https://github.com/Chris1221/wgba/issues",
      "Source": "https://github.com/Chris1221/wgba",
   },
   author='Chris Cole',
   author_email='ccole@well.ox.ac.uk',
   install_requires = ["pyBigWig", "pandas"],
   packages=['wgba', 'wgba.sizes'],  
   include_package_data = True,
   entry_points={
        'console_scripts': [
            'wgba = wgba.cli:cli',
        ],
    }
)


import argparse
from .genomes import add_build
from .formats import check_bigwig, check_bed

from enum import Enum, auto


def cli():
    parser = argparse.ArgumentParser(
        description="Figure out which genome build is likely from a particular file"
    )
    parser.add_argument("path", help="File", nargs="+")
    parser.add_argument(
        "-f",
        "--format",
        choices=["bed", "bigwig", "auto", "add_build"],
        default="auto",
        help="",
    )
    parser.add_argument("-s", "--summary", default=False, action="store_true")
    parser.add_argument(
        "-c",
        "--check",
        default=False,
        action="store_true",
        help="Check if all files are on the same build.",
    )
    parser.add_argument("-t", "--tol", default=2, type=int)

    args = parser.parse_args()

    builds = []
    for file in args.path:
        if args.format == "auto":
            format = detect_filetype(file)
        else:
            format = args.format

        d = vars(args)
        d["path"] = file
        if format == "bigWig":
            builds.append(check_bigwig(**vars(args)))
        elif format == "bed":
            builds.append(check_bed(**vars(args)))
        elif format == "add_build":
            add_build(file)

    if args.check:
        all = [item for sublist in builds for item in sublist]
        if len(set(all)) == 1:
            print("Hooray! Your files are all on the same build.")
        else:
            print("\nWARNING: Your files are not all on the same build.")


# Do this later
class FileTypes(Enum):
    BED = auto()
    BIGWIG = auto()


def detect_filetype(path):
    if any([i in path.lower() for i in [".bed"]]):
        return "bed"
    elif any([i in path.lower() for i in [".bigWig", ".bigwig", ".bw"]]):
        return "bigWig"

    return NotImplementedError(
        "Please specify the format of your file(s) using the -f flag."
    )
